<?

 /******************************************************************************************
 * Filename: script.php
 *
 * This file is part of the Achievements JAS plugin.
 * Copyright 2003-2017 by Matthijs Schuurman
 *
 * The Achievements JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/achievements/
 ******************************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["include"]."script.php");

 class Achievements_Script extends Script
 {
  function __construct()
  {
   parent::__construct();

   $this->blacklist[]="_data"; //update blacklist
   $this->blacklist[]="_mathConvert";
  }

  function clear()
  {
   parent::clear();

   $this->e->clear();
  }

  function execute($name)
  {
   $r=parent::execute($name);

   if($r===true)
    $r=0;

   return $r;
  }


  /*
   * Achievements
   */

  function achievement($achievement)
  {
   if($this->progress($achievement)>=1)
    return true;

   return false;
  }

  function progress($achievement)
  {
   $d=Data::cache("achievements_progress", "view", "all");

   $d->setFilter("usernr='".$this->get("usernr")."' AND achievementnr.name='".$achievement."'");

   $d->setShowFields("progress");

   if(!$d->get()||!$d->next())
   {
    $this->e->add($d->e);
    return false;
   }

   return $d->v["progress"];
  }


  /*
   * Data
   */

  function value($module, $filter=false, $show)
  {
   global $_config;

   $d=Data::cache($module, "view", "all");

   $d->clearValues();
   $d->setValue("show_total", 1);

   $d->setShowFields($show);

   if($filter)
    $d->setFilter($filter);

   $_config->su(true);
   if($d->e->any("fatal")||!$d->get())
   {
    $_config->su();

    $this->e->add($d->e);
    return false;
   }

   $_config->su();

   if(!$d->next())
    return false;

   return $d->v[$show];
  }


  function rows($module, $filter=false)
  {
   global $_config;

   $d=Data::cache($module, "view", "all");

   $d->clearValues();

   $d->setShowFields();

   if($filter)
    $d->setFilter($filter);

   $_config->su(true);
   if($d->e->any("fatal")||!$d->get(false, true))
   {
    $_config->su();

    $this->e->add($d->e);
    return false;
   }

   $_config->su();

   return $d->getValue("show_count");
  }


  function average($module_or_values, $filter=false, $field=false, $interval=false)
  {
   if(is_array($module_or_values))
    return (float)array_sum($module_or_values)/sizeof($module_or_values);

   return $this->_aggregation($module_or_values, $filter, __FUNCTION__, $field, $interval);
  }
  function min($module_or_values, $filter=false, $field=false, $interval=false)
  {
   if(is_array($module_or_values))
    return min($module_or_values);

   return $this->_aggregation($module_or_values, $filter, __FUNCTION__, $field, $interval);
  }
  function max($module_or_values, $filter=false, $field=false, $interval=false)
  {
   if(is_array($module_or_values))
    return max($module_or_values);

   return $this->_aggregation($module_or_values, $filter, __FUNCTION__, $field, $interval);
  }
  function sum($module_or_values, $filter=false, $field=false, $interval=false)
  {
   if(is_array($module_or_values))
    return array_sum($module_or_values);

   return $this->_aggregation($module_or_values, $filter, __FUNCTION__, $field, $interval);
  }
  function count($module_or_values, $filter=false, $field=false, $interval=false)
  {
   if(is_array($module_or_values))
    return sizeof($module_or_values);

   return $this->_aggregation($module_or_values, $filter, __FUNCTION__, $field, $interval);
  }
  function _aggregation($module, $filter, $type, $field=false, $interval=false)
  {
   global $_config;

   $d=Data::cache($module, "view", "all");

   if($owner=$d->check("properties", "owner"))
    $owner=$owner[0];

   $d->setValue("aggregation_type", $type);

   if($field)
    $d->setValue("aggregation_field", $field);
   else
    $d->setValue("aggregation_field", $owner);

   if($interval)
    $d->setValue("aggregation_interval", $interval);

   $show=false;
   $ignoreTypes=Array("date", "time", "datetime", "rights");
   if($fields=$d->check("use", "sql"))
   {
    foreach($fields as $field)
    {
     if($field==$owner)
      continue;
     if($d->check("properties", "identifier", $field))
      continue;
     if(in_array($d->field[$field]["type"], $ignoreTypes))
      continue;

     $show=$field;
     break;
    }
   }

   if(!$show)
   {
    $this->e->add("Can't determine show field for module: ".$module, "error");
    return false;
   }

   $d->setShowFields($show);

   if($filter)
    $d->setFilter($filter);

   $_config->su(true);

   if($d->e->any("fatal")||!$d->get())
   {
    $_config->su();

    $this->e->add($d->e);
    return false;
   }

   $_config->su();

   if($d->vmax==0)
    return 0;
   else if($d->vmax==1)
    return $d->field[$show]["value"][0];

   return $d->field[$show]["value"];
  }



  /*
   * Math
   */

  function _mathConvert(&$left, &$right)
  {
   if(is_array($left)||is_array($right))
    return false;

   if(!preg_match("/^\-?[0-9\.]+$/", $left)) //nan
    $left=$this->get($left);
   if(!preg_match("/^\-?[0-9\.]+$/", $right)) //nan
    $right=$this->get($right);

   return true;
  }

  function add($left, $right=0)
  {
   if(!$this->_mathConvert($left, $right))
    return false;

   return $left+$right;
  }
  function subtract($left, $right)
  {
   if(!$this->_mathConvert($left, $right))
    return false;

   return $left-$right;
  }

  function multiply($left, $right)
  {
   if(!$this->_mathConvert($left, $right))
    return false;

   return $left*$right;
  }
  function divide($left, $right)
  {
   if(!$this->_mathConvert($left, $right))
    return false;

   return $left/$right;
  }
 }
?>
